
#include "stdafx.h"
#include "Windows.h"
#include "uart.h"
#include "wdt03.h"

BYTE bCOM     = 0;
BYTE bAddress = 0;
WORD wT;   // Time used

// *************************************************************
// iCOM   : COM port number, 1~255
// dwBaud : Baud rate
// iModuleAddress : WDT-03 Module Address, 0~255
WORD WDT03_Initialize(int iCOM, DWORD dwBaud, int iModuleAddress)
{
	if ( iModuleAddress > 255 || iModuleAddress < 1 )
		return WDT03_InvalidModuleAddress;
	if ( bCOM ) WDT03_Close();

	WORD wRtn;
	wRtn = Open_Com( (BYTE)iCOM, dwBaud, 8, 0, 0 );  // Data format=8N1
	if ( wRtn != 0 )
		return wRtn;

	bCOM     = (BYTE) iCOM;
	bAddress = (BYTE) iModuleAddress;

	return 0;
}

// *************************************************************
WORD WDT03_ClearAllDO()
{
	if ( !bCOM ) return WDT03_ComPortNotOpen;

	WORD wRtn;
	char szCmd[20], szReceive[20];

	sprintf(szCmd, "#%02d0000", bAddress);
	wRtn = Send_Receive_Cmd(bCOM, szCmd, szReceive, DEFAULT_TIMEOUT, DEFAULT_CHECKSUM, &wT);
	if ( wRtn != 0 )
		return wRtn;
	
	return 0;
}

// *************************************************************
// iChannel : Channel number for D/O, 0~2
// iValue   : 0=Off, 1=On
WORD WDT03_SetDO(int iChannel, int iValue)
{
	if ( !bCOM ) return WDT03_ComPortNotOpen;
	if ( iChannel > 2 || iChannel < 0 ) return WDT03_InvalidChannelNumber;
	if ( iValue   > 1 || iValue   < 0 ) return WDT03_InvalidValue;

	WORD wRtn;
	char szCmd[10], szReceive[20];

	sprintf(szCmd, "#%02d1%d%02d", bAddress, iChannel, iValue);
	wRtn = Send_Receive_Cmd(bCOM, szCmd, szReceive, DEFAULT_TIMEOUT, DEFAULT_CHECKSUM, &wT);
	if ( wRtn != 0 )
		return wRtn;

	return 0;
}

// *************************************************************
// iReadBackValue : Return the D/O read-back value for all channels
WORD WDT03_ReadBackAllDO(int *iReadBackValue)
{
	if ( !bCOM ) return WDT03_ComPortNotOpen;

	WORD wRtn;
	char szCmd[10], szReceive[20];
	int  iValue;

	*iReadBackValue = 0;
	sprintf(szCmd, "$%02d6", bAddress);
	wRtn = Send_Receive_Cmd(bCOM, szCmd, szReceive, DEFAULT_TIMEOUT, DEFAULT_CHECKSUM, &wT);
	if ( wRtn != 0 )
		return wRtn;

	if ( szReceive[0] != '!' )
		return WDT03_InvalidResult;

	szReceive[3] = 0;
	iValue = atoi(szReceive +1);
	if ( iValue < 0 || iValue > 7 )
		return WDT03_InvalidResult;
	
	*iReadBackValue = iValue;
	return 0;
}

// *************************************************************
// iChannel : D/O channel number, 0~2
// iValue   : The D/O read-back value for the specified channel
WORD WDT03_ReadBackDO(int iChannel, int *iValue)
{
	if ( !bCOM ) return WDT03_ComPortNotOpen;
	if ( iChannel > 2 || iChannel < 0 ) return WDT03_InvalidChannelNumber;

	*iValue = 0;

	int  iAllValue;
	WORD wRtn = WDT03_ReadBackAllDO(&iAllValue);
	if ( wRtn != 0 )
		return wRtn;

	*iValue = (iAllValue >> iChannel) & 0x01;
	return 0;
}

// *************************************************************
// iChannel: Temp. channel number, 0~2
// iValue: The Temp. value for the specified channel
WORD WDT03_GetTemp(int iChannel, int *iValue)
{
	if ( !bCOM ) return WDT03_ComPortNotOpen;
	if ( iChannel > 2 || iChannel < 0 ) return WDT03_InvalidChannelNumber;

	WORD wRtn;
	char szCmd[10], szReceive[20];
	char *stopstring;

	*iValue = 0;
	sprintf(szCmd, "~%02d7%d", bAddress, iChannel +5); // Temp-0 at channel 5
	wRtn = Send_Receive_Cmd(bCOM, szCmd, szReceive, DEFAULT_TIMEOUT, DEFAULT_CHECKSUM, &wT);
	if ( wRtn != 0 )
		return wRtn;

	if ( szReceive[0] != '!' )
		return WDT03_InvalidResult;

	*iValue = strtol(szReceive+1, &stopstring, 16);
	return 0;
}

// *************************************************************
void WDT03_Close()
{
	if ( bCOM ) 
	{
		Close_Com( bCOM );
		bCOM = 0;
	}
}

